/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.source.Annotation;

import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;

/**
 * An <code>Annotation</code> which spans an entire line and holds an object associated
 * with the line in an <code>IEntityContainer</code>.
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class LineAnnotation<T> extends Annotation implements IEntityContainer<T> {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(LineAnnotationModel.class);

	/** Constant <code>TYPE_GENERIC="Annotation.TYPE_UNKNOWN"</code> */
	public static final String TYPE_GENERIC = Annotation.TYPE_UNKNOWN;

	private T entity;
	private ILineDisplayStrategy lineDisplayStrategy;

	private boolean dirty = false;
	private boolean markedAsMerged;
	private boolean markedAsNew;
	private T mergeTarget;
	private DeleteConfiguratorBase configurator= null;
	private boolean markAsDelete;


	/**
	 * <p>Constructor for LineAnnotation.</p>
	 *
	 * @param entity a T object.
	 * @param lineDisplayStrategy a {@link eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy} object.
	 * @param <T> a T object.
	 */
	public LineAnnotation(T entity, ILineDisplayStrategy lineDisplayStrategy) {
		this.entity = entity;
		this.lineDisplayStrategy = lineDisplayStrategy;
		setType(TYPE_GENERIC);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.Annotation#getText()
	 */
	/** {@inheritDoc} */
	@Override
	public String getText() {
		return getEditableText();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.Annotation#setText(java.lang.String)
	 */
	/** {@inheritDoc} */
	@Override
	public void setText(String text) {
		lineDisplayStrategy.setText(text, entity);
		dirty  = true;
		super.setText(text);
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		// For debugging
		return getText();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getAttachedEntities()
	 */
	/**
	 * <p>getAttachedEntities</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	@Override
    public Set getAttachedEntities() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getEntity()
	 */
	/**
	 * <p>Getter for the field <code>entity</code>.</p>
	 *
	 * @return a T object.
	 */
	@Override
    public T getEntity() {
		return entity;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getEditableText()
	 */
	/**
	 * <p>getEditableText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getEditableText() {
		return lineDisplayStrategy.getText(entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getMergeTarget()
	 */
	/**
	 * <p>Getter for the field <code>mergeTarget</code>.</p>
	 *
	 * @return a T object.
	 */
	@Override
    public T getMergeTarget() {
		return mergeTarget;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsDeleted()
	 */
	/**
	 * <p>isMarkedAsDeleted</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean isMarkedAsDeleted() {
		return super.isMarkedDeleted();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsMerged()
	 */
	/**
	 * <p>isMarkedAsMerged</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean isMarkedAsMerged() {
		return markedAsMerged;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsNew()
	 */
	/**
	 * <p>isMarkedAsNew</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean isMarkedAsNew() {
		return markedAsNew;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsDeleted()
	 */
	/**
	 * <p>markAsDeleted</p>
	 */
	@Override
    public void markAsDeleted() {
		super.markDeleted(true);
		if (!isMarkedAsMerged()) {
			lineDisplayStrategy.setStatusMessage(entity + " deleted.", entity);
		}
	}
	 public void markAsDeleted(DeleteConfiguratorBase config) {
			super.markDeleted(true);
			this.configurator = config;
			if (!isMarkedAsMerged()) {
				lineDisplayStrategy.setStatusMessage(entity + " deleted.", entity);
			}
		}
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsMerged(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	/**
	 * <p>markAsMerged</p>
	 *
	 * @param mergeTarget a T object.
	 */
	@Override
    public void markAsMerged(T mergeTarget) {
		this.mergeTarget = mergeTarget;
		markedAsMerged = true;
		lineDisplayStrategy.setStatusMessage("'" + entity + "' merged into '" + mergeTarget + "'.", entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsNew()
	 */
	/** {@inheritDoc} */
	@Override
    public void markAsNew(boolean isNew) {
		markedAsNew = isNew;
		lineDisplayStrategy.setStatusMessage("New entity created.", entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#setDirty(boolean)
	 */
	/** {@inheritDoc} */
	@Override
    public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isDirty()
	 */
	/**
	 * <p>isDirty</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean isDirty() {
		return dirty;
	}

	/**
	 * <p>getHoverText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getHoverText() {
		return lineDisplayStrategy.getHoverText(entity);
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityContainer#setEntity(java.lang.Object)
     */
    @Override
    public void setEntity(Object entity) {
        this.entity = (T) entity;
    }

	public DeleteConfiguratorBase getDeleteConfigurator() {
		return this.configurator;
	}

	public void setDeleteConfigurator(DeleteConfiguratorBase config) {
		this.configurator = config;

	}
}
