/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

/**
 * Gives a different color and font style to all text identified as supplemental text by
 * its corresponding LineAnnotation.
 *
 * Note: implementing IPartitionScanner in a straightforward fashion proved buggy, so this more obtuse
 * extension of RuleBasedPartitionScanner should be used instead. Uses a matching IRule whose pattern changes
 * depending on the current line.
 *
 * @author p.ciardelli
 * @created 28.10.2009
 * @see CopyOfSupplementalTextScanner
 * @version 1.0
 */
public class SupplementalTextScanner extends RuleBasedPartitionScanner {
	private static final Logger logger = Logger
			.getLogger(SupplementalTextScanner.class);
	
	private ILineDisplayStrategy lineDisplayStrategy;
	private Document document;
	private ISourceViewer sourceViewer;

	private Map<Integer, String> supplementalTexts;
	private SupplementalTextRule rule;
	private int lastLine;
	
	/**
	 * <p>Constructor for SupplementalTextScanner.</p>
	 *
	 * @param sourceViewer a {@link org.eclipse.jface.text.source.ISourceViewer} object.
	 * @param lineDisplayStrategy a {@link eu.etaxonomy.taxeditor.annotatedlineeditor.ILineDisplayStrategy} object.
	 */
	public SupplementalTextScanner(ISourceViewer sourceViewer, ILineDisplayStrategy lineDisplayStrategy) {
		this.sourceViewer = sourceViewer;
		this.lineDisplayStrategy = lineDisplayStrategy;
		
		TextAttribute textAttribute = new TextAttribute(Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GRAY), null, SWT.ITALIC);
		IToken supplementalTextToken = new Token(textAttribute);
		
		rule = new SupplementalTextRule(supplementalTextToken);
		setPredicateRules(new IPredicateRule[] {rule});
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.rules.RuleBasedPartitionScanner#setPartialRange(org.eclipse.jface.text.IDocument, int, int, java.lang.String, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void setPartialRange(IDocument document, int offset, int length,
			String contentType, int partitionOffset) {
		
		// Get all supplemental texts in the partial range
		supplementalTexts = new HashMap<Integer, String>();
		LineAnnotationModel model = (LineAnnotationModel) sourceViewer.getAnnotationModel();
		if(model != null){
			Iterator iter = model.getAnnotationIterator(offset, length, true, true);
			while (iter.hasNext()) {
				Object next = iter.next();
				if (next instanceof LineAnnotation) {
					 Object entity = ((LineAnnotation) next).getEntity();
					 String supplementalText = lineDisplayStrategy.getSupplementalText(entity);
					 if (supplementalText != null && supplementalText.length() > 0) {
						 try {
							int line = document.getLineOfOffset(model.getPosition((LineAnnotation) next).getOffset());
							supplementalTexts.put(line, supplementalText);
						} catch (BadLocationException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						 
					 }
				}
			}
		}
		this.document = (Document) document; 
		super.setPartialRange(document, offset, length, contentType, partitionOffset);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.rules.RuleBasedPartitionScanner#nextToken()
	 */
	/** {@inheritDoc} */
	@Override
	public IToken nextToken() {
		try {
			int line = document.getLineOfOffset(getTokenOffset());
			if (line != lastLine) {
				rule.setStartSequence(supplementalTexts.get(line));
			}
			lastLine = line;
		} catch (BadLocationException e) {
			logger.debug("Ignoring", e);
		}
		return super.nextToken();
	}
	
	class SupplementalTextRule extends SingleLineRule {

		static final String DUMMY_SEQUENCE = "_dummy_sequence";
		
		/**
		 * @param startSequence
		 * @param endSequence
		 * @param token
		 * @param escapeCharacter
		 * @param breaksOnEOL
		 */
		public SupplementalTextRule(IToken token) {
			super(DUMMY_SEQUENCE, null, token, '\\', true);
		}
				
		/**
		 * Set pattern match for rule.
		 * 
		 * @param sequence
		 */
		protected void setStartSequence(String sequence) {
			if (sequence != null && sequence.length() > 0) {
				fStartSequence = sequence.toCharArray();
			}
		}
		
	}
}
