/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.ReferenceCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.IdentifiableEntitySortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.ReferenceSortProvider;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * <p>ReferenceEditorInput class.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 */
public class ReferenceEditorInput extends AbstractBulkEditorInput<Reference> {

	/**
	 *
	 */
	private static final long serialVersionUID = 3806243517765126749L;

	/** Constant <code>ID="bulkeditor.input.reference"</code> */
	public static final String ID = "bulkeditor.input.reference";

	private static ReferenceEditorInput instance;

	/**
	 * <p>getID</p>
	 *
	 * @return the iD
	 */
	public static String getID() {
		return ID;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	/**
	 * <p>getName</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getName() {
		return BulkEditorInputType.REFERENCE.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	/**
	 * <p>getToolTipText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getToolTipText() {
		return getName();
	}

	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new ReferenceEditorInput();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean isMergingEnabled() {
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMarkerTypeEditingEnabled(eu.etaxonomy.cdm.model.common.MarkerType)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean isMarkerTypeEditingEnabled(MarkerType markerType) {
		if (MarkerType.PUBLISH().equals(markerType)) {
			return true;
		}
		return super.isMarkerTypeEditingEnabled(markerType);
	}


	/** {@inheritDoc} */
	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.getSearchManager().findReferences(configurator);
	}

	// Entity persistence

	/** {@inheritDoc} */
	@Override
	public Reference loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{"*", "inReference.*", "authorTeam.*"});
		return CdmStore.getService(IReferenceService.class).load(entityUuid, propertyPaths);
	}

	/** {@inheritDoc}
	 * @throws ReferencedObjectUndeletableException */
	@Override
    public boolean delete(Reference entity, DeleteConfiguratorBase config) throws ReferencedObjectUndeletableException {
	    if (entity.getUuid() != null){
	        return CdmStore.getService(IReferenceService.class).delete(entity.getUuid()) != null;
	    }else{
	        return false;
	    }
	}

	/** {@inheritDoc} */
	@Override
    public Reference save(Reference entity) {
	   return CdmStore.getService(IReferenceService.class).merge(entity, true).getMergedEntity();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#createEntityCreator()
	 */
	@Override
	protected IEntityCreator<Reference> createEntityCreator() {
		return new ReferenceCreator();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getSortProviders()
	 */
	@Override
	public List<IBulkEditorSortProvider<Reference>> getSortProviders() {
		List<IBulkEditorSortProvider<Reference>> sortProviders = super.getSortProviders();

		sortProviders.add(0, new IdentifiableEntitySortProvider<Reference>());
		sortProviders.add(1, new ReferenceSortProvider());

		return sortProviders;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#getTypeText(java.lang.Object)
	 */
	@Override
	public String getTypeText(Object entity) {
		if(entity instanceof Reference){
			return ((Reference) entity).getType().getMessage(CdmStore.getDefaultLanguage());
		}
		return super.getTypeText(entity);
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {

    }

}
